/*!
 * $Id: app.js 53238 2017-11-10 12:22:37Z sseiz $
 * Copyright Zeta Software GmbH
 */
/* jshint strict: false, multistr: true, smarttabs:true, jquery:true, devel:true */

document.documentElement.className = document.documentElement.className.replace(/no-js/g, 'js');

var nualc = navigator.userAgent.toLowerCase();

$z.support.placeholder = (function(){
    var i = document.createElement('input');
    return 'placeholder' in i;
})();

function zpTextOverflow(e, filler){
	filler = typeof filler !== 'undefined' ? filler : "…";
	
	$z(e).each(function(d){
		var wordcount = 0;
		var nodes = $z(this).find("*");
		var nodecount = nodes.length;
		var done = false;
		
		while ( !done && $z(this).overflown() && nodecount >= 0 ){
			//console.log(Date.now() + " Article No.:" + d + " Nodecount: " + nodecount);
			if ( nodecount > 0 ){
				var chtml = $z(nodes).eq(nodecount-1).html();
				var target = $z(nodes).eq(nodecount-1);
			}
			else{
				var chtml = $z(this).html();
				var target = $z(this);
			}
			var words = chtml.split(" ");
			
			for (i = words.length; i >= 0; i--){
				if ( i == words.length ){ // only runs this in the first iteration of the for loop
					// schnell an die richtige Wortanzahl rantasten, wenn mehr als 7 wörter durschsucht werden müssen.
					while ( i >=2 && $z(this).overflown() ){
						i = parseInt(i / 2);
						//console.log("    " + Date.now() + " Wordsestimate: " + i);
						$z(target).html(words.slice(0,i).join(" ") + filler);
					}
					i = i * 2;
				}
				//console.log("    Wordcount: " + i );
				if ( i > 0 ) {
					$z(target).html(words.slice(0,i).join(" ") + filler);
				}
				else{
					$z(target).remove();
				}
				if (!$z(this).overflown()){
					done = true;
					break;
				}
			}
			nodecount--;
		}
	});
};

//
// Debounce calls to "callback" routine so that multiple calls
// made to the event handler before the expiration of "delay" are
// coalesced into a single call to "callback". Also causes the
// wait period to be reset upon receipt of a call to the
// debounced routine.
//
function zpdebounce(delay, callback) {
    if (typeof(delay)==='undefined') delay = 250;
    var timeout = null;
    return function () {
        //
        // if a timeout has been registered before then
        // cancel it so that we can setup a fresh timeout
        //
        if (timeout) {
            clearTimeout(timeout);
        }
        var args = arguments;
        timeout = setTimeout(function () {
            callback.apply(null, args);
            timeout = null;
        }, delay);
    };
}
//
// Throttle calls to "callback" routine and ensure that it
// is not invoked any more often than "delay" milliseconds.
// http://blogorama.nerdworks.in/javascriptfunctionthrottlingan/
//
function zpthrottle(delay, callback) {
	if (typeof(delay)==='undefined') delay = 250;
    var timeout = null;
    var previousCall = new Date().getTime();
    return function() {
        var time = new Date().getTime();

        //
        // if "delay" milliseconds have expired since
        // the previous call then propagate this call to
        // "callback"
        //
        if ((time - previousCall) >= delay) {
            previousCall = time;
            callback.apply(null, arguments);
            
             // make sure callback is called again in case event ended before timer elapsed
	    	if (timeout) {
				clearTimeout(timeout);
			}
			timeout = setTimeout(function () {
				callback.apply(null, arguments);
				timeout = null;
			}, delay);
        }
    };
}

function trace(s) {
  try { console.log(s); } catch (e) { alert(s); }
}

// test for touch device
function is_touch_device() {
	//return !!('ontouchstart' in window) || !!(navigator.msMaxTouchPoints) || !!(navigator.maxTouchPoints) || window.DocumentTouch && document instanceof DocumentTouch ;

	// use same logic as jquery.flexslider.js
	//var msGesture = window.navigator && window.navigator.msPointerEnabled && window.MSGesture,
    //touch = (( "ontouchstart" in window ) || msGesture || window.DocumentTouch && document instanceof DocumentTouch);
	var touch = 'ontouchstart' in window || navigator.maxTouchPoints;
	return touch;
}

// Helper for Nav-Menues with hover effects to make them work via alternating clicks
// 1st Click will open the Submenues, 2nd Click will load the link associated with the clicked element
function hoverToClickMenu(element, breakpointMobileMenu, instancenumber, triangleMode) { 
	//var listenEvent = 'ontouchend' in document.documentElement ? "touchend" : "click";
	var listenEvent = "click";
		
	// The stock browser on Android 4.X can't cancel touchend events and will thus always fire an additional click event, so we need to revert to click events StS 2015-02-24
	if ( nualc.indexOf("android 4") > -1 && nualc.indexOf("chrome") === -1 ) {
		listenEvent = "click";
	}
	
	/* clean up added styles after the user resizes the browser window and might reach desktop resolution */
	if ( instancenumber == 1 && breakpointMobileMenu !== undefined  ) {
			var menuResizeTimer;
			
			var clearAddedStyles = function(){
				if (  $z(window).width() > parseInt(breakpointMobileMenu) ){					
					$z(".hoverToClickMenuAdded").children("ul").css({'display' : '', 'visibility' : ''});
					$z(".hoverToClickMenuAdded").removeClass("clicked").removeClass("open").removeClass("hoverToClickMenuAdded");
				}
			};
			
			$z(window).resize(function(e) {
				clearTimeout(menuResizeTimer);
				menuResizeTimer = setTimeout(clearAddedStyles, 250);
			});
	}

	var firstClick = function(element, event) {
		var event = event || window.event;
		if ( breakpointMobileMenu !== undefined && $z(window).width() <= parseInt(breakpointMobileMenu) ){
			// switch triangleMode on in Mobile Menues where we always show the triangles
			triangleMode = true;
		}
		else if ( !is_touch_device() && breakpointMobileMenu !== undefined && listenEvent == "click" && $z(window).width() > parseInt(breakpointMobileMenu) ) {
			// we're NOT displaying a mobile menu on non-touch devices, so return and don't modify the click behavior
			return true;
		}
		
		if ( triangleMode ){
			if ( (event.pageX - $z(element).offset().left) <= parseInt($z(element).css("padding-left")) || (event.pageX - $z(element).offset().left) > (parseInt($z(element).css("padding-left")) + $z(element).width() -6) ){
				// user clicked on triangle to the left or right of a link
				var hasVisibleChilds = $z(element).parent().children("ul").css("display") == "block" && $z(element).parent().children("ul").css("visibility") == "visible";
				//console.log("HasvisibleChilds: " + hasVisibleChilds);
				if ( hasVisibleChilds && ($z(element).parent().hasClass("open") ||  $z(element).parent().hasClass("active")) ){
					$z(element).parent().removeClass("clicked").removeClass("open");
					$z(element).parent().children("ul").css({'display' : 'none', 'visibility' : ''});
				}
				else{
					$z(element).parent().addClass("hoverToClickMenuAdded").addClass("clicked").addClass("open");
					$z(element).parent().children("ul").css({'display' : 'block', 'visibility' : 'visible'});
				}
				event.preventDefault();
				return false;
			}	
			else{
				// user clicked directly on link, so we load the url immediately
				return true;
			}
		}
		else{
			var otherMenus = $z(element).parent().prevAll(".clicked").add($z(element).parent().nextAll(".clicked"));
			otherMenus.removeClass("clicked").removeClass("open");
			otherMenus.find("ul").css({'display' : '', 'visibility' : ''});
			otherMenus.find(".clicked").removeClass("clicked");
			otherMenus.find(".open").removeClass("open");
			var hasVisibleChilds = $z(element).parent().children("ul").css("display") == "block" && $z(element).parent().children("ul").css("visibility") == "visible";
		
			if ( $z(element).parent().hasClass("clicked") || hasVisibleChilds ){ // TODO ZP13 check layouts for incompatibilities due to commenting out this: || ($z(element).parent().children("xul").css("display") == "block" && $z(element).parent().children("xul").css("visibility") == "visible") ) {
				// element has been clicked before, so now we fire a click
				return true;
			}
			// element has been clicked for the first time, so we do not fire a click and only show submenues
		
			// add ".open" classname to parent li element so we can style it if we want
			$z(element).parent().addClass("clicked").addClass("open");
			// in case suckerfish is used
			$z(element).parent().children("ul").css({'display' : 'block', 'visibility' : 'visible'});
			return false;
		}
	};
	$z(element).on( listenEvent , function(event) {
		var firstClickResult = firstClick($z(this), event);
		if ( !firstClickResult ){
			event.preventDefault();
		}
		return firstClickResult;
	});
}

// For IE8 and earlier version which don't have native support for Date.now.
if (!Date.now) {
  Date.now = function() {
    return new Date().valueOf();
  };
}

// polyfill for array.find (native substitute for underscrote _.find
// https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/find
if (!Array.prototype.find && Object.defineProperty) {
  Object.defineProperty(Array.prototype, 'find', {
	value: function(predicate) {
	 'use strict';
	 if (this == null) {
	   throw new TypeError('Array.prototype.find called on null or undefined');
	 }
	 if (typeof predicate !== 'function') {
	   throw new TypeError('predicate must be a function');
	 }
	 var list = Object(this);
	 var length = list.length >>> 0;
	 var thisArg = arguments[1];
	 var value;

	 for (var i = 0; i < length; i++) {
	   value = list[i];
	   if (predicate.call(thisArg, value, i, list)) {
		 return value;
	   }
	 }
	 return undefined;
	}
  });
}

// FIX Viewports for iOS Devices as they always report the short side for "device-width", even when in landscape
/*
if (navigator.userAgent.match(/xxiPhone/i) || navigator.userAgent.match(/xxiPad/i)) {
	var viewportmeta = $z('meta[name=viewport]'); //document.querySelector('meta[name="viewport"]');
	if (viewportmeta) {
		// set viewport on first page load if in portrait orientation
		if ( window.orientation !== 0 && window.orientation !== 180 ){
			viewportmeta.attr("content", "width=device-height, initial-scale=1.0");
		}
		// add eventListener so viewport is set when device is rotated after the initial page load
		document.addEventListener('orientationchange', function () {
			if ( window.orientation !== 0 && window.orientation !== 180 ){
				viewportmeta.attr("content", "width=device-height, initial-scale=1.0");
			}
			else{
				viewportmeta.attr("content", "width=device-width, initial-scale=1.0");
			}
		}, false);
	}
}
*/

// lets us define css classes which only apply after all assets are loaded
$z(window).load(function(){
	$z("body").addClass("loaded");
});
				
$z(document).ready(function () {
	if ( $z(".zpgrid").length === 0 ){
		// we have a layout that was not modified for ZP13 i.e. a copied Layout from Version 12.5 or earlier
		// layouts not modified have no div with class .zpgrid, so we append the class to the document body
		$z('.zparea[data-areaname="Standard"], .zparea[data-areaname="Banner"], .zparea[data-areaname="Footer"]').addClass("zpgrid copiedlayout");
	}

	//define some helper css classes
		$z("html").removeClass("no-js");
		$z("html, body").addClass("js");
		// recognize IE (since IE10 doesn't support conditional comments anymore)
		// removed in jQuery 1.9, so be careful!
		if ( $z.browser.msie || !!navigator.userAgent.match(/Trident.*rv\:11\./) ) {
			$z("html").removeClass("notie");
			$z("html").addClass("ie");
			$z("html").addClass("ie" + parseInt($z.browser.version, 10));
		}
		else if( !!navigator.userAgent.match(/Edge\/\d\d/) ){
			$z("html").addClass("edge");
		}
		else if ($z.browser.mozilla){
			$z("html").addClass("mozilla");
		}
		else if ($z.browser.safari && !navigator.appVersion.match(/Chrome\//) ){
			$z("html").addClass("safari");
			if ( navigator.appVersion.match(/version\/(\d+)/i) ){
				$z("html").addClass("safari" + navigator.appVersion.match(/version\/(\d+)/i)[1]);
			}
		}
		
		// detect object-fit css support
		if ( document.createElement("detect").style.objectFit === "" ) {
			$z("body").addClass("objectfit");
		}
		
		// system prefers reduced motion (used i.e. to switch off animations in css)
		if ( window.matchMedia && window.matchMedia( "(prefers-reduced-motion)" ).matches ){
			$z("body").addClass("zpreducemotion");
		}
		
		// add a top-padding to html5 audio because firefox has a time indicator implemented as a bubble which would be cut off due to our overflow hidden grid system
		if ($z.browser.mozilla){
			$z("audio").animate({paddingTop: '+=12px'}, 0); // we only use .animate() here, as that is a convenient way to be able to add values to (possibly) existing values
		}
		
		if(is_touch_device()) {
			// add .touch class to body if we run on a touch device, so we can use the class in css (used e.g. in ONLINE-CMS)
			$z("body").removeClass("notouch");
			$z("body").addClass("touch");
			
			// fix for hover menues (which contain submenues) to make them work on touch devices
			var breakpointmobilemenu = $z(".clickhovermenu").data("breakpointmobilemenu");
			var trianglemode = $z(".clickhovermenu").data("trianglemode") || false;
			$z(".touchhovermenu li:has(li) > a").each(function(i){
				hoverToClickMenu(this, breakpointmobilemenu, i, trianglemode);
			});
		}
		else{
			// In case we want to substitute hover with click menues on non touch devices too
			$z("body").removeClass("touch");
			$z("body").addClass("notouch");
			var breakpointmobilemenu = $z(".clickhovermenu").data("breakpointmobilemenu");
			var trianglemode = $z(".clickhovermenu").data("trianglemode") || false;
			$z(".clickhovermenu li:has(li) > a").each(function(i){
				hoverToClickMenu(this, breakpointmobilemenu, i, trianglemode);
			});
		}
		
		// set correct dimensions for breakout elements which in CSS are only approximated due to problems with browsers handling scrollbars differently when using vw/vh
		function setBreakout(){
			var bodyWidth = $z("body").outerWidth();
			$z(".supportsbreakout body:not(.withnews) .zpBreakout:not(.hasNews)").css("width",bodyWidth+"px").css("padding-left", "0").css("margin-left","calc(-" + bodyWidth/2 + "px + 50% )");
			$z("body.supportsbreakout:not(.withnews) .zpBreakout:not(.hasNews)").css("width",bodyWidth+"px").css("padding-left", "0").css("margin-left","calc(-" + bodyWidth/2 + "px + 50% )");
		}
		setBreakout();
		$z(window).resize(zpdebounce(100, function(event) {
			setBreakout();
		}));
		
		// animate Containers on Scroll
		function isElementInViewport (el) {
			//special bonus for those using jQuery
			if (el instanceof jQuery) {
				el = el[0];
			}
			
			var rect = el.getBoundingClientRect();

			//document.documentElement.clientHeight is a fallback for IE8
			return ( (rect.top >= 0 && rect.top <= (window.innerHeight || document.documentElement.clientHeight) ) || (rect.bottom >= 0 && rect.bottom <= (window.innerHeight || document.documentElement.clientHeight)) );
		}
		var lastScrollTop = 0;
		var hasAnimatableContent = $z("body:not(.zpreducemotion) .zpanimate").length;
		if ( hasAnimatableContent ){
			// wrap animatable content in wrapper with overflow hidden to prevent scrollbars due to content shifted to the right
			$z('body:not(.zpreducemotion) .zpanimate.slideleft, body:not(.zpreducemotion) .zpanimate.floatin, body:not(.zpreducemotion) .zpanimate.zoomout').each(function(i, el){
				var inlineStyles = "width:" + $z(this).css("width") + ";";
				inlineStyles += "padding-left:" + $z(this).css("padding-left") + ";";
				inlineStyles += "margin-left: auto;";
				inlineStyles += "margin-right: auto;";
				var currentElement = $z(this);
				// remove margins from current element since they'll be applied to wrapper by the zpContainer/zpRow class css
				currentElement.css("margin-top", "0px");
				currentElement.css("margin-bottom", "0px");
				// depending if current Element is zpRow or zpContainer, use propper className in wrapper too, so the correct CSS applies to the wrapper
				var currentElementClass = "zpContainer";
				if ( currentElement.hasClass("zpRow") ){
					currentElementClass = "zpRow";
				}
				if ( currentElement.hasClass("floatin") ){
					currentElementClass += " floatin";
				}
				// remove inline-scripts from ZP search-page as these destroy the dom during animation
				var currentScript = currentElement.find('script.zpsearchinlinescript').text();
				currentElement.find('script.zpsearchinlinescript').remove();
				currentElement.wrap('<div class="zpanimatewrap ' + currentElementClass + '" style="' + inlineStyles + '">');
				// unwrap the content again after the animation finished to make sure it properly displays and doesn't get disturbed by the wrapper
				currentElement.one("webkitTransitionEnd otransitionend oTransitionEnd msTransitionEnd transitionend", function(event) {
					if ( currentElement.parent().hasClass("zpanimatewrap") ){
						currentElement.unwrap();
					}
					// reset margins
					currentElement.css("margin-top", "");
					currentElement.css("margin-bottom", "");
				});
			}); ;
			
			function doAnimations(e){
				var delay = 0;
				var st = $z(this).scrollTop();
				if (st > lastScrollTop){
					var scrollDirection = "down";
				}
				else{
					var scrollDirection = "up";
				}
				lastScrollTop = st;
				$z("body:not(.zpreducemotion) .zpanimate").each(function(i, el){
					var viewportTestElement = el;
					// FYI: el is a Dom-Object, not a jQuery-Object
					if ( el.className.indexOf("floatin") !== -1 ){
						// Elements with Effect "Einschweben" have a translateY(50%) applied, so we need to get the proper .top coordinates of the parent-wrapper in order to find out if it's in viewport
						viewportTestElement = el.parentElement;
					}
					
					if ( isElementInViewport(viewportTestElement) ) {
						if ( ! $z(this).hasClass("show")){
							var currentElement = $z(this);
							// animate consecutively visible elements via setTimeout, so the elements are animated consecutive and not at the same time 
							setTimeout(function() { 
								currentElement.addClass("show");
							}, delay);
							delay += 250;
						}
					}
					else{
						// reset delay once we reach an element out of viewport, so once this reaches viewport, it isn't animated with delay
						delay = 0;
					}
				});
			}
			
			$z(window).off('load.zpanim scroll.zpanim');
			$z(window).on('load.zpanim', doAnimations); 
			$z(window).on('scroll.zpanim', zpthrottle(50, doAnimations)); 
		}
});

// define zp Namespace for later use in individual widgets
var zp = {  
}; // end zp

// test HTML5 field-type support and store it in zp.html5support
zp.html5support = {number: false, email: false, tel: false, url: false, date: false, time: false, color: false, search: false};
var tester = document.createElement('input');
for(var i in zp.html5support){
	// Use try/catch because IE9 throws "Invalid Argument" when trying to use unsupported input types
	try {
		tester.type = i;
		if(tester.type === i){
			zp.html5support[i] = true;
		}
	} catch (e){}
}

(function($){
	// make $z.unique also work on arrays and not only DOM-Elements (without this, we have a problem with the EventCalendars in Chrome)
	// http://stackoverflow.com/a/7366133
    var _old = $.unique;
    $.unique = function(arr){
        // do the default behavior only if we got an array of elements
        if ( arr.length == 0  || !!arr[0].nodeType){
            return _old.apply(this,arguments);
        }
        else {
            // reduce the array to contain no dupes via grep/inArray
            return $.grep(arr,function(v,k){
                return $.inArray(v,arr) === k;
            });
        }
    };
    
    // custom function to shorten and suffix text with _filler_ to make it fit its container - StS 2016-10-27
    // call it like this: $z(".mySelector").fitText();
    $.fn.fitText = function( filler ){
    	filler = typeof filler !== 'undefined' ? filler : "…";
    	
		zpTextOverflow(this, filler);
	
		return this;
	}
	
	// custom function to find out if content of an element is overflowing 
	$.fn.overflown=function(direction){
		if (typeof(direction)==='undefined') direction = "any";
		var e=this[0];
		var buffer = 4; // needs to be greater than value + buffer to be considered overflown. Compensates Browser rounding-bugs(cough, IE)…
		if ( typeof e !== "undefined" ){
			switch(direction) {
				case "x":
					return e.scrollWidth>(e.clientWidth+buffer);
					break;
				case "y":
					return e.scrollHeight>(e.clientHeight+buffer);
					break;
				default:
					return e.scrollHeight>(e.clientHeight+buffer)||e.scrollWidth>(e.clientWidth+buffer);
			}
		}
		else{
			return false;
		}
	}
})($z);

/*!
 * END $Id: app.js 53238 2017-11-10 12:22:37Z sseiz $ 
 */


